import moyasar
import json
from moyasar.constructor import Constructor


class Create(Constructor):
    def __init__(self, data):
        super().__init__(**data)


    @classmethod
    def create_url(cls):
        return f'{moyasar.resource_url(cls.__name__)}'.lower()

    @classmethod
    def create(cls, data):
        response = moyasar.request('POST', cls.create_url(), data)
        response = json.loads(response.text)
        return cls(response)



import moyasar
import tests.server_stubs as ss
import tests.fixtures.fixtures as f
import copy
import pytest


def test_create_should_return_intiated_payment_for_sadad_source():
    params = {"amount": 1000, "currency": "SAR", "description": "Test"}
    ss.stub_server_request("post", moyasar.Payment.create_url(),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment = moyasar.Payment.create(params)
    assert isinstance(payment, moyasar.Payment)
    assert payment.status == "initiated"
    assert int(payment.amount) == params['amount']
    assert payment.currency == params['currency']
    assert payment.description == params['description']


def test_create_with_amount_less_than_100_cent_should_raise_validation_errror():
    modified_resource = copy.deepcopy(f.payment)
    modified_resource.update({"amount": 90})
    ss.stub_server_request('post', moyasar.Payment.create_url(), resource=modified_resource,
                           status=400, error_message="Validation Failed",
                           errors={"amount": "must be greater than 99"})
    assert pytest.raises(Exception)


def test_create_payment_for_inovice_should_be_acceptable():
    ss.stub_server_request("get", moyasar.Invoice.list_url(),
                           resource=f.invoices, status=200)
    invoices = moyasar.Invoice.list()
    first_invoice_id = ''
    for key, val in invoices[0].__dict__.items():
        if key == "id":
            first_invoice_id = val

    modified_resource = copy.deepcopy(f.payment)
    modified_resource.update({"invoice_id": first_invoice_id})
    ss.stub_server_request("post", moyasar.Payment.create_url(),
                           resource=modified_resource, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment = moyasar.Payment.create(modified_resource)
    assert isinstance(payment, moyasar.Payment)
    assert payment.invoice_id == first_invoice_id


def test_list_should_return_list_of_payment_objects():
    ss.stub_server_request("get", moyasar.Payment.list_url(),
                           resource=f.payments, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payments = moyasar.Payment.list()
    assert type(payments) == list
    assert isinstance(payments[0], moyasar.Payment)


def test_find_should_return_payemnt_object_if_id_is_correct():
    id = '328f5dca-91ec-435d-b13f-86052a1d0f7b'
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment = moyasar.Payment.fetch(id)
    assert payment.id == id


def test_update_should_update_payment_description():
    # fetch to update
    id = '328f5dca-91ec-435d-b13f-86052a1d0f7b'
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment = moyasar.Payment.fetch(id)
    # update fetched payment
    ss.stub_server_request("put", f'{moyasar.api_url}/payments/{payment.id}',
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    new_description = {"description": "Test"}
    updated = payment.update(new_description)
    # simulate updated record
    ss.stub_server_request("post", moyasar.Payment.create_url(),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    after_updated = moyasar.Payment.create(new_description)

    assert updated.status_code == 200
    assert new_description["description"] == after_updated.description


# def test_refund_should_operate_normally_by_recharging_back_all_amount_as_a_default():
#     id = '328f5dca-91ec-435d-b13f-86052a1d0f7b'
#     ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
#                            resource=f.payment, status=200)
#     moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
#     original = moyasar.Payment.fetch(id)
#     params = {'status': 'refunded', 'refunded': original.amount}
#     custom_invoice = copy.deepcopy(f.payment)
#     custom_invoice.update(params)
#     ss.stub_server_request("post", f'{moyasar.api_url}/payments/{id}/refund',
#                            resource=custom_invoice, status=200)
#     moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
#     refunded = original.refund(amount=original.amount)


def test_refund_should_accept_partial_refund_amounts():
    pass


def test_refund_with_failed_payment_should_raise_invalid_request_error():
    pass


def test_eqaulity_check_holds_among_identical_payments_only():
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment_one = moyasar.Payment.fetch(id)
    ss.remove_stub()
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment_two = moyasar.Payment.fetch(id)

    payment_one_values = []
    payment_two_values = []
    for key, val in payment_one.__dict__.items():
        if key != 'source':
            payment_one_values.append(val)
        if key == 'source':
            for k, v in val.__dict__.items():
                payment_one_values.append(v)
    for key, val in payment_two.__dict__.items():
        if key != 'source':
            payment_two_values.append(val)
        if key == 'source':
            for k, v in val.__dict__.items():
                payment_two_values.append(v)
    assert payment_one_values == payment_two_values


def test_eqaulity_check_differentiate_non_identical_payments():
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=f.payment, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment_one = moyasar.Payment.fetch(id)
    ss.remove_stub()
    modified_resource = copy.deepcopy(f.payment)
    modified_resource.update({"amount": "5000"})
    ss.stub_server_request("get", moyasar.Payment.fetch_url(id),
                           resource=modified_resource, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    payment_two = moyasar.Payment.fetch(id)

    payment_one_values = []
    payment_two_values = []
    for key, val in payment_one.__dict__.items():
        if key != 'source':
            payment_one_values.append(val)
        if key == 'source':
            for k, v in val.__dict__.items():
                payment_one_values.append(v)
    for key, val in payment_two.__dict__.items():
        if key != 'source':
            payment_two_values.append(val)
        if key == 'source':
            for k, v in val.__dict__.items():
                payment_two_values.append(v)
    assert payment_one_values != payment_two_values


import os
import py
import pytest
import moyasar
import tests.server_stubs as ss
import tests.test_helper as th
import json
import tests.fixtures.fixtures as f


def test_that_it_has_a_version_number():
    assert moyasar.api_version is not ''


def test_should_accept_api_key():
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    assert moyasar.api_key is not ''


# def test_request_should_throw_exception_if_key_is_nil():
#     ss.stub_server_request(method='GET', url=moyasar.api_url + '/payments',
#                            resource=f.payments, status=401)
#     moyasar.api_key = ''
#     with pytest.raises(Exception):
#         moyasar.request('GET', moyasar.api_url + '/payments', None)
#
#
# def test_request_return_success_when_correct_key_given():
#     ss.stub_server_request(method='GET', url=moyasar.api_url + '/payments',
#                            resource=f.payments)
#     moyasar.api_key = ''
#     response = moyasar.request('GET', moyasar.api_url + '/payments', None)
#     assert response.status_code == 200


def test_request_should_return_json_object():
    ss.stub_server_request(method='GET', url=moyasar.api_url + '/payments',
                           resource=f.payments, status=200)
    response = moyasar.request('GET', moyasar.api_url + '/payments', None)
    assert isinstance(response.json(), dict)



import moyasar
import json


class Refund:
    def refund_url(self, id):
        return f'{moyasar.resource_url(self.__class__.__name__)}/{id}/refund'.lower()

    def refund(self, amount=None):
        data = None
        if amount is not None:
            data = {'amount': amount}

        response = moyasar.request('POST', self.refund_url(self.id), data)
        response = json.loads(response.text)
        moyasar.fill_object(self, response)



import moyasar


class Update:
    def update_url(self, id):
        return f'{moyasar.resource_url(self.__class__.__name__)}/{id}'.lower()

    def update(self, data):
        response = moyasar.request('PUT', self.update_url(self.id), data)
        return response


import base64
import os
import pytest
import json
import httpretty

API_URI = r"api(mig)?\.moyasar\.com\/v1\/.*"


def stub_server_request(method, url, resource, status=200, error_message= None, errors= None):
    response = ''
    prepare_json = json.dumps(resource)
    if status in range(200, 201):
        response = json.loads(prepare_json)
    elif status in range(400, 429):
        response = {"type": "authentication_error", "message": "Invalid authorization credentials", "errors": None, "http_code": status}
        if status == 401:
            response.update({"type": "authentication_error"})
        if status == 400:
            response.update({"type": "invalid_request_error", "message": error_message, "errors": errors})

    httpretty.enable()
    httpretty.reset()
    httpretty.register_uri(method.upper(), url, body=json.dumps(response), status=status)


def remove_stub():
    httpretty.disable()



import requests
import json
from moyasar.payment import Payment
from moyasar.invoice import Invoice

api_key = None
api_version = 'v1'
api_url = f'https://api.moyasar.com/{api_version}'


def resource_url(resource_name):
    return f'{api_url}/{resource_name}s'


def fill_object(object, data):
    for key in data:
        object.__setattr__(key, data[key])


def request(http_verb, url, data, key=None):
    moyasar_key = key or api_key

    if moyasar_key is None:
        raise Exception('API key must be provided')

    request = {
        'method': http_verb.upper(),
        'url': url,
        'auth': (api_key, ''),
        'headers': {
            'Content-Type': 'application/json'
        }
    }
    if data is not None:
        if http_verb.upper() == 'GET':
            request['params'] = data
        else:
            request['data'] = json.dumps(data)

    res = requests.request(**request)
    if 400 <= res.status_code <= 404:
        json_string = res.text
        json_dict = json.loads(json_string)
        json_dict["http_code"] = res.status_code
        raise Exception(f'{json.dumps(json_dict)}')
    if 500 <= res.status_code <= 504:
        raise Exception(f'API Error with status code: {res.status_code}')
    return res



from moyasar.actions.fetch import Fetch
from moyasar.actions.list import List
from moyasar.actions.create import Create
from moyasar.actions.update import Update


class Resource(Fetch, List, Create, Update):
    pass



from moyasar.resource import Resource
from moyasar.actions.refund import Refund
from moyasar.constructor import Constructor


class Source(Constructor):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def build(cls, source):
        if source['type'] == "creditcard":
            source = Source.source_to_creditcard(source)
        else:
            source = Source.source_to_sadad(source)
        return source

    @classmethod
    def source_to_creditcard(cls, data):
        data.pop('type')
        return CreditCard(**data)

    @classmethod
    def source_to_sadad(cls, data):
        data.pop('type')
        return Sadad(**data)


class CreditCard(Source):
    pass


class Sadad(Source):
    pass


class Payment(Resource, Refund):
    def __init__(self, data):
        super().__init__(data)
        self.source = Source.build(self.source)

    def refund(self, amount=None):
        super().refund(amount)
        self.source = Source.build(self.source)
        return self



import moyasar
import json


class Cancel:
    def cancel_url(self, id):
        return f'{moyasar.resource_url(self.__class__.__name__)}/{id}/cancel'.lower()

    def cancel(self):
        response = moyasar.request('PUT', self.cancel_url(self.id), None)
        response = json.loads(response.text)
        moyasar.fill_object(self, response)
        return response



import os
import json
import pytest
import tests.server_stubs as ss
import moyasar
import tests.fixtures.fixtures as f
import copy


def test_create_should_return_intiated_invoice_for_sadad_source():
    params = {"amount": 1000, "currency": "SAR", "description": "Test"}
    ss.stub_server_request("post", moyasar.Invoice.create_url(),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice = moyasar.Invoice.create(params)
    assert isinstance(invoice, moyasar.Invoice)
    assert invoice.status == "initiated"
    assert int(invoice.amount) == params['amount']
    assert invoice.currency == params['currency']
    assert invoice.description == params['description']


def test_list_should_return_list_of_invoice_objects():
    ss.stub_server_request("get", moyasar.Invoice.list_url(),
                           resource=f.invoices, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoices = moyasar.Invoice.list()
    assert type(invoices) == list
    assert isinstance(invoices[0], moyasar.Invoice)


def test_find_should_return_invoice_object_if_id_is_correct():
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get", moyasar.Invoice.fetch_url(id),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice = moyasar.Invoice.fetch(id)
    assert invoice.id == id


def test_update_should_update_invoice_description():
    # fetch to update
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get",  moyasar.Invoice.fetch_url(id),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice = moyasar.Invoice.fetch(id)
    # update fetched invoice
    ss.stub_server_request("put", f'{moyasar.api_url}/invoices/{invoice.id}',
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    new_description = {"description": "Test"}
    updated = invoice.update(new_description)
    # simulate updated record
    ss.stub_server_request("post", moyasar.Invoice.create_url(),
                          resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    after_updated = moyasar.Invoice.create(new_description)

    assert updated.status_code == 200
    assert new_description["description"] == after_updated.description


def test_cancel_should_return_invoice_object_if_id_is_correct():
    # copy resource to prepare a custom version for this case
    cancel_resource = copy.deepcopy(f.invoice)
    cancel_resource.update({"status": "canceled"})
    id = '1b92356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get",  moyasar.Invoice.fetch_url(id),
                           resource=cancel_resource, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice = moyasar.Invoice.fetch(id)
    ss.stub_server_request("put", f'{moyasar.api_url}/invoices/{invoice.id}/cancel',
                           resource=cancel_resource, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    canceled = invoice.cancel()
    assert canceled["status"] == "canceled"


def test_eqaulity_check_holds_among_identical_invoices_only():
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get",  moyasar.Invoice.fetch_url(id),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice_one = moyasar.Invoice.fetch(id)
    ss.remove_stub()
    ss.stub_server_request("get",  moyasar.Invoice.fetch_url(id),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice_two = moyasar.Invoice.fetch(id)
    invoice_one_values = []
    for val in invoice_one.__dict__.values():
        invoice_one_values.append(val)
    invoice_two_values = []
    for val in invoice_two.__dict__.values():
        invoice_two_values.append(val)
    assert invoice_one_values == invoice_two_values


def test_eqaulity_check_differentiate_non_identical_invoices():
    id = '1b82356d-b5fd-46f8-bde9-3680d62f289a'
    ss.stub_server_request("get", moyasar.Invoice.fetch_url(id),
                           resource=f.invoice, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice_one = moyasar.Invoice.fetch(id)
    ss.remove_stub()
    modified_resource = copy.deepcopy(f.invoice)
    modified_resource.update({"amount": "5000"})
    ss.stub_server_request("get", moyasar.Invoice.fetch_url(id),
                           resource=modified_resource, status=200)
    moyasar.api_key = 'sk_test_BQokikJOvBiI2HlWgH4olfQ2'
    invoice_two = moyasar.Invoice.fetch(id)
    invoice_one_values = []
    for val in invoice_one.__dict__.values():
        invoice_one_values.append(val)
    invoice_two_values = []
    for val in invoice_two.__dict__.values():
        invoice_two_values.append(val)
    assert invoice_one_values != invoice_two_values



import moyasar
import json
from moyasar.constructor import Constructor


class Fetch(Constructor):
    def __init__(self, data):
        super().__init__(data)

    @classmethod
    def fetch_url(cls, id):
        return f'{moyasar.resource_url(cls.__name__)}/{id}'.lower()

    @classmethod
    def fetch(cls, id):
        response = moyasar.request('GET', cls.fetch_url(id), None)
        response = json.loads(response.text)
        return cls(response)



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="moyasar",
    version="0.6.0",
    author="Moyasar",
    author_email="developers@moyasar.com",
    description="Moyasar Python language wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/moyasar/moyasar-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests',
        'httpretty',
        'pytest'
    ],
    download_url='https://github.com/moyasar/moyasar-python/archive/v0.6.0.tar.gz'
)



import moyasar
import json
from moyasar.constructor import Constructor


class List(Constructor):
    def __init__(self, data):
        super().__init__(data)

    @classmethod
    def list_url(cls):
        return f'{moyasar.resource_url(cls.__name__)}'.lower()

    @classmethod
    def list(cls, data=None):
        response = moyasar.request('GET', cls.list_url(), data)
        response = json.loads(response.text)
        field_name = f'{cls.__name__}s'.lower()
        resource_list = []
        for resource in response[field_name]:
            resource_list.append(cls(resource))

        return resource_list
